;;; navi2ch-board.el --- subject list module for navi2ch

;; Copyright (C) 2000 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: network, 2ch

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

(require 'navi2ch-article)
(require 'navi2ch-board-misc)
(require 'navi2ch-net)
(require 'cl)

(defvar navi2ch-board-mode-map nil)
(unless navi2ch-board-mode-map
  (setq navi2ch-board-mode-map (copy-keymap navi2ch-bm-mode-map))
  ;; (define-key navi2ch-board-mode-map "q" 'navi2ch-board-exit)
  (define-key navi2ch-board-mode-map "s" 'navi2ch-board-sync)
  (define-key navi2ch-board-mode-map "r" 'navi2ch-board-select-view-range)
  (define-key navi2ch-board-mode-map "a" 'navi2ch-board-add-bookmark)
  (define-key navi2ch-board-mode-map "d" 'navi2ch-board-hide-article)
  (define-key navi2ch-board-mode-map "h" 'navi2ch-board-toggle-hide)
  (define-key navi2ch-board-mode-map "b" 'navi2ch-board-toggle-bookmark)
  (define-key navi2ch-board-mode-map "w" 'navi2ch-board-write-message)
  (define-key navi2ch-board-mode-map "2" 'navi2ch-board-two-pain)
  (define-key navi2ch-board-mode-map "A" 'navi2ch-board-add-global-bookmark)
  (define-key navi2ch-board-mode-map "\C-c\C-f" 'navi2ch-article-find-file)
  (define-key navi2ch-board-mode-map "e" 'navi2ch-board-expire))

(defvar navi2ch-board-mode-menu-spec
  (navi2ch-bm-make-menu-spec
   "Board"
   '(["Sync" navi2ch-board-sync]
     ["Add bookmark" navi2ch-board-add-bookmark]
     ["Add global bookmark" navi2ch-board-add-global-bookmark]
     ["Hide line" navi2ch-board-hide-article])))

(defvar navi2ch-board-regexp-list
  (list "^\\([0-9]+\\)\\.dat,\\([^(\n]*\\)(\\([0-9]+\\))\n"
        "^\\([0-9]+\\)\\.dat<>\\([^<\n]*\\)<\\([0-9]+\\)>\n"
        "^\\([0-9]+\\)\\.dat,\\([^$B!J(B(\n]*\\)[($B!J(B]\\([0-9]+\\)[$B!K(B)]\n"))

(defvar navi2ch-board-current-board nil)
(defvar navi2ch-board-subject-list nil)

;; bookmark mode
(defvar navi2ch-board-bookmark-mode nil)
(defvar navi2ch-board-bookmark-mode-map nil)
(unless navi2ch-board-bookmark-mode-map
  (setq navi2ch-board-bookmark-mode-map (make-sparse-keymap))
  (define-key navi2ch-board-bookmark-mode-map "d"
    'navi2ch-board-delete-bookmark)
  (define-key navi2ch-board-bookmark-mode-map "a" 'undefined))

;; hide mode
(defvar navi2ch-board-hide-mode nil)
(defvar navi2ch-board-hide-mode-map nil)
(unless navi2ch-board-hide-mode-map
  (setq navi2ch-board-hide-mode-map (make-sparse-keymap))
  (define-key navi2ch-board-hide-mode-map "d"
    'navi2ch-board-cancel-hide)
  (define-key navi2ch-board-hide-mode-map "a" 'undefined))

;;; navi2ch-bm callbacks
(defun navi2ch-board-set-property (begin end item)
  (put-text-property begin end 'article item))

(defun navi2ch-board-get-property (point)
  (get-text-property point 'article))

(defun navi2ch-board-get-article (item)
  item)

(defun navi2ch-board-get-board (item)
  navi2ch-board-current-board)

(defun navi2ch-board-exit ()
  (navi2ch-board-save-info))
 
;;; navi2ch-board functions
(defsubst navi2ch-board-get-uri (board)
  "$B8e$m$N(B / $B$,IU$$$?(B uri $B$rJV$9!#(B
kako $B$J$i$P$=$l$KBP1~$7$?(B uri $B$K$9$k(B"
  (let ((uri (cdr (assq 'uri board))))
    (when (string-match "[^/]$" uri)
      (setq uri (concat uri "/")))
    (concat uri (cdr (assq 'kako board)))))

(defsubst navi2ch-board-get-url (board &optional file-name)
  (concat (navi2ch-board-get-uri board)
          (or file-name "subject.txt")))

(defsubst navi2ch-board-get-file-name (board &optional file-name)
  (navi2ch-expand-file-name
   (let ((uri (navi2ch-board-get-uri board)))
     (string-match "http://\\(.+\\)" uri)
     (setq uri (match-string 1 uri))
     (when (string-match ".+kako/" uri)
       (setq uri (match-string 0 uri)))
     (concat uri
             (or file-name "subject.txt")))))

(defsubst navi2ch-board-get-matched-article ()
  "match $B$7$?7k2L$+$i(B article $B$rF@$k(B"
  (let ((id (match-string 1))
        (str (match-string 2))
        (num (match-string 3))
        article)
    (setq str (navi2ch-replace-string "^ +" "" str)
          str (navi2ch-replace-string " +$" "" str)
          str (navi2ch-replace-html-tag str))
    (list (cons 'subject str)
          (cons 'response num)
          (cons 'artid id))))

(defun navi2ch-board-get-subject-list (file)
  "file $B$+$i%9%l$N(B list $B$r:n$k!#(B"
  (when (file-exists-p file)
    (with-temp-buffer
      (navi2ch-insert-file-contents file)
      (goto-char (point-min))
      (let ((regexp (navi2ch-board-regexp-test))
            ;; (file-list (directory-files
            ;;             (navi2ch-board-get-file-name
            ;;              navi2ch-board-current-board "") nil "\\.dat$"))
            list)
        (while (re-search-forward regexp nil t)
          (setq list (cons (navi2ch-board-get-matched-article) list)))
        (nreverse list)))))

(defun navi2ch-board-regexp-test ()
  (save-excursion
    (beginning-of-line)
    (catch 'loop
      (dolist (regexp navi2ch-board-regexp-list)
        (when (looking-at regexp)
          (throw 'loop regexp))))))


(defun navi2ch-board-insert-subjects (list)
  (let ((prev (point))
        (bookmark (cdr (assq 'bookmark navi2ch-board-current-board)))
        (hide (cdr (assq 'hide navi2ch-board-current-board)))
        (i 1))
    (dolist (article list)
      (when (cond (navi2ch-board-bookmark-mode
                   (member (cdr (assq 'artid article)) bookmark))
                  (navi2ch-board-hide-mode
                   (member (cdr (assq 'artid article)) hide))
                  (t
                   (not (member (cdr (assq 'artid article)) hide))))
        (navi2ch-bm-insert-subject
         article i
         (cdr (assq 'subject article))
         (format "(%4s)" (cdr (assq 'response article)))))
      (setq i (1+ i)))))

(defun navi2ch-board-select-board (board &optional force)
  (navi2ch-board-mode)
  (navi2ch-bm-setup 'navi2ch-board)
  (setq navi2ch-board-bookmark-mode nil)
  (save-excursion
    (if (eq navi2ch-board-current-board board)
        (navi2ch-board-sync force)
      (setq navi2ch-board-current-board board)
      (run-hooks 'navi2ch-board-select-board-hook)
      (navi2ch-board-sync force 'first))))

(defun navi2ch-board-setup-menu ()
  (easy-menu-define navi2ch-board-mode-menu
		    navi2ch-board-mode-map
		    "Menu used in navi2ch-board"
		    navi2ch-board-mode-menu-spec)
  (easy-menu-add navi2ch-board-mode-menu))
  
(defun navi2ch-board-mode ()
  "\\{navi2ch-board-mode-map}"
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'navi2ch-board-mode)
  (setq mode-name "Navi2ch Board")
  (setq buffer-read-only t)
  (use-local-map navi2ch-board-mode-map)
  (navi2ch-board-setup-menu)
  (run-hooks 'navi2ch-board-mode-hook)
  (force-mode-line-update))

;; (defun navi2ch-board-exit ()
;;   (interactive)
;;   (dolist (x (navi2ch-article-buffer-list))
;;     (when x
;;       (delete-windows-on x)))
;;   (navi2ch-board-save-info)
;;   (when (get-buffer navi2ch-board-buffer-name)
;;     (delete-windows-on navi2ch-board-buffer-name)
;;     (bury-buffer navi2ch-board-buffer-name))
;;   (when navi2ch-list-buffer-name
;;     (let ((win (get-buffer-window navi2ch-list-buffer-name)))
;;       (if win
;; 	  (select-window win)
;; 	(switch-to-buffer navi2ch-list-buffer-name)))))

(defun navi2ch-board-sync (&optional force first)
  (interactive "P")
  (save-excursion
    (let* ((buffer-read-only nil)
           (navi2ch-net-force-update (or navi2ch-net-force-update
                                         force))
           (board navi2ch-board-current-board)
           (url (navi2ch-board-get-url board))
           (file (navi2ch-board-get-file-name board))
           time header)
      (when first
        (setq board (navi2ch-board-load-info)))
      (unless navi2ch-offline
        (setq header (navi2ch-net-update-file
                      url file (cdr (assq 'time board))
                      nil navi2ch-board-max-line))
        (setq time (cdr (assoc "Last-Modified" header)))
        (when time
          (setq board
                (navi2ch-put-alist 'time time
                                   board)
                navi2ch-board-current-board board)))
      (when (or first time)
        (erase-buffer)
        (setq navi2ch-board-subject-list
              (navi2ch-board-get-subject-list file))
        (navi2ch-board-insert-subjects navi2ch-board-subject-list)
        (navi2ch-board-save-info)
        (navi2ch-board-set-mode-line))))
  (run-hooks 'navi2ch-board-after-sync-hook))

(defun navi2ch-board-set-mode-line ()
  (let* ((board navi2ch-board-current-board)
         (host (cdr (assq 'uri board))))
    (string-match "http://\\([^/]+\\)" host)
    (setq host (match-string 1 host))
    (setq navi2ch-mode-line-identification
          (format "%s (%s)" (cdr (assq 'name board)) host))
    (navi2ch-set-mode-line-identification)))

(defun navi2ch-board-write-message ()
  (interactive)
  (navi2ch-message-write-message navi2ch-board-current-board nil t))

(defun navi2ch-board-two-pain ()
  (interactive)
  (let* ((list-buf (get-buffer navi2ch-list-buffer-name))
         (board-buf (get-buffer navi2ch-board-buffer-name))
         (art-buf (navi2ch-article-current-buffer))
         (list-win (get-buffer-window (or list-buf "")))
         (art-win (get-buffer-window (or art-buf "")))
         buf)
    (when board-buf
      (delete-other-windows)
      (switch-to-buffer board-buf)
      (setq buf
            (cond ((and list-buf art-buf)
                   (cond ((and list-win art-win) art-buf)
                         (list-win art-buf)
                         (art-win list-buf)
                         (t art-buf)))
                  (list-buf list-buf)
                  (art-buf art-buf)))
      (when buf
        (if (eq buf list-buf)
            (split-window-horizontally navi2ch-list-window-width)
          (split-window-vertically navi2ch-board-window-height)
          (other-window 1))
        (switch-to-buffer buf))
      (other-window 1))))
  

(defun navi2ch-board-save-info ()
  (let ((board navi2ch-board-current-board))
    (when (and board
               (eq (navi2ch-get-major-mode navi2ch-board-buffer-name)
                   'navi2ch-board-mode))
      (navi2ch-save-info
       (navi2ch-board-get-file-name
        board "board.info")
       (list (assq 'bookmark board)
             (assq 'hide board)
             (assq 'time board))))))
   

(defun navi2ch-board-load-info ()
  (let ((alist (navi2ch-load-info
                (navi2ch-board-get-file-name
                 navi2ch-board-current-board "board.info"))))
    (dolist (x alist)
      (setq navi2ch-board-current-board
            (navi2ch-put-alist (car x) (cdr x)
                               navi2ch-board-current-board)))
    navi2ch-board-current-board))

(defun navi2ch-board-select-view-range ()
  (interactive)
  (setq-default navi2ch-article-view-range
                (navi2ch-article-select-view-range-subr)))
        

(defun navi2ch-board-add-global-bookmark ()
  (interactive)
  (let ((article (get-text-property (point) 'article)))
    (if article
        (navi2ch-bookmark-add
         navi2ch-board-current-board
         article)
      (message "Can't select this line!"))))

(defun navi2ch-board-delete-line (sym func msg)
  (let ((artid (cdr (assq 'artid (get-text-property (point) 'article))))
        (list (cdr (assq sym navi2ch-board-current-board))))
    (if artid
        (progn
          (setq list (funcall func artid list))
          (setq navi2ch-board-current-board
                (navi2ch-put-alist sym list
                                   navi2ch-board-current-board))
          (let ((buffer-read-only nil))
            (delete-region (save-excursion (beginning-of-line) (point))
                           (save-excursion (forward-line) (point))))
          (message msg))
      (message "can't select this line!"))))

;;; from Wanderlust(wl-expire.el)
(defsubst navi2ch-board-expire-date-p (key-datevec file)
  (let ((datevec (condition-case nil
                     (navi2ch-make-datevec (nth 5 (file-attributes file)))
                   (error nil))))
    (and
     datevec (> (aref datevec 1) 0)
     (string<
      (navi2ch-make-sortable-date datevec)
      (navi2ch-make-sortable-date key-datevec)))))

(defun navi2ch-board-expire (&optional board ask)
  (interactive)
  (and (interactive-p) (setq ask t))
  (or board (setq board navi2ch-board-current-board))
  (let ((dir (navi2ch-board-get-file-name board "")))
    (when (and (file-exists-p dir)
	       (or (not ask)
		   (y-or-n-p "Expire current borad?")))
      (when navi2ch-board-expire-date
	(let ((list (directory-files dir t "\\.dat$"))
	      (key-datevec (navi2ch-get-offset-datevec
			    (navi2ch-make-datevec (current-time))
			    navi2ch-board-expire-date)))
	  (dolist (file list)
	    (when (navi2ch-board-expire-date-p key-datevec file)
	      (string-match "\\([0-9]+\\)\\.dat$" file)
	      (let ((info (concat dir "info/" (match-string 1 file))))
		(when (file-exists-p info)
		  (delete-file info)))
	      (delete-file file))))))))
  
;;; bookmark mode
(navi2ch-set-minor-mode 'navi2ch-board-bookmark-mode
                        " Bookmark"
                        navi2ch-board-bookmark-mode-map)

(defun navi2ch-board-add-bookmark ()
  (interactive)
  (unless navi2ch-board-bookmark-mode
    (let ((artid (cdr (assq 'artid (get-text-property (point) 'article))))
          (list (cdr (assq 'bookmark navi2ch-board-current-board))))
      (if artid
          (unless (member artid list)
            (setq list (cons artid list))
            (setq navi2ch-board-current-board
                  (navi2ch-put-alist 'bookmark list
                                     navi2ch-board-current-board))
            "Add bookmark")
        (message "can't select this line!")))))

(defun navi2ch-board-delete-bookmark ()
  (interactive)
  (navi2ch-board-delete-line 'bookmark 'delete
                             "Delete bookmark"))

(defun navi2ch-board-toggle-bookmark ()
  (interactive)
  (setq navi2ch-board-bookmark-mode
        (not navi2ch-board-bookmark-mode)
        navi2ch-board-hide-mode nil)
  (force-mode-line-update)
  (let ((buffer-read-only nil))
    (save-excursion
      (erase-buffer)
      (navi2ch-board-insert-subjects navi2ch-board-subject-list))))

;;; hide mode
(navi2ch-set-minor-mode 'navi2ch-board-hide-mode
                        " Hide"
                        navi2ch-board-hide-mode-map)

(defun navi2ch-board-hide-article ()
  (interactive)
  (navi2ch-board-delete-line 'hide
                             (lambda (artid list)
                               (if (member artid list)
                                   list
                                 (cons artid list)))
                             "Hide article"))

(defun navi2ch-board-cancel-hide ()
  (interactive)
  (navi2ch-board-delete-line 'hide 'delete
                             "Cancel hide article"))

(defun navi2ch-board-toggle-hide ()
  (interactive)
  (setq navi2ch-board-hide-mode
        (not navi2ch-board-hide-mode)
        navi2ch-board-bookmark-mode nil)
  (force-mode-line-update)
  (let ((buffer-read-only nil))
    (save-excursion
      (erase-buffer)
      (navi2ch-board-insert-subjects navi2ch-board-subject-list))))

(provide 'navi2ch-board)

;;; navi2ch-board.el ends here