;;; navi2ch-bookmark.el --- global bookmark module for navi2ch

;; Copyright (C) 2001 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: network, 2ch

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; 

;;; Code:

(require 'cl)
(require 'navi2ch-board-misc)

(defvar navi2ch-bookmark-mode-map nil)
(unless navi2ch-bookmark-mode-map
  (setq navi2ch-bookmark-mode-map
        (copy-keymap navi2ch-bm-mode-map))
  ;; (define-key navi2ch-bookmark-mode-map "q" 'navi2ch-bookmark-exit)
  (define-key navi2ch-bookmark-mode-map "d" 'navi2ch-bookmark-delete)
  (define-key navi2ch-bookmark-mode-map "\C-k" 'navi2ch-bookmark-cut)
  (define-key navi2ch-bookmark-mode-map "\C-y" 'navi2ch-bookmark-yank))
  
(defvar navi2ch-bookmark-mode-menu-spec
  (navi2ch-bm-make-menu-spec
   "Bookmark"
   nil))

(defvar navi2ch-bookmark-list nil "bookmark $B$rI=$9%j%9%H(B")
(defvar navi2ch-bookmark-file (navi2ch-expand-file-name
                               "bookmark2.txt"))

(defvar navi2ch-bookmark-cut-stack nil)
(defvar navi2ch-bookmark-board
  '((name . "$B%V%C%/%^!<%/(B")
    (type . bookmark)
    (id . "bmark")))

;;; navi2ch-bm callbacks
(defun navi2ch-bookmark-set-property (begin end item)
  (put-text-property begin end 'item item))

(defun navi2ch-bookmark-get-property (point)
  (get-text-property point 'item))

(defun navi2ch-bookmark-get-article (item)
  (cdr (assq 'article (cdr (assoc item navi2ch-bookmark-list)))))

(defun navi2ch-bookmark-get-board (item)
  (cdr (assq 'board (cdr (assoc item navi2ch-bookmark-list)))))

(defun navi2ch-bookmark-exit ())

;;; navi2ch-bookmark functions
(defun navi2ch-bookmark-exist (board article)
  (assoc (concat (cdr (assq 'uri board))
		 (cdr (assq 'artid article)))
	 navi2ch-bookmark-list))

(defun navi2ch-bookmark-add (board article)
  "BOARD $B$H(B ARTICLE $B$GI=$5$l$k(B $B%9%l%C%I$rDI2C(B"
  (let* ((item (concat (cdr (assq 'uri board))
                     (cdr (assq 'artid article)))))
    (setq navi2ch-bookmark-list
          (navi2ch-put-alist
           item (list
                 (cons 'board board)
                 (cons 'article article))
           navi2ch-bookmark-list)))
  (message "Add current article to global bookmark."))

(defun navi2ch-bookmark-insert-subject (num item)
  (navi2ch-bm-insert-subject
   item num
   (cdr (assq 'subject (navi2ch-bookmark-get-article item)))
   (format "[%s]" (cdr (assq 'name (navi2ch-bookmark-get-board item))))))

(defun navi2ch-bookmark-insert-subjects ()
  (let ((i 1))
    (dolist (x navi2ch-bookmark-list)
      (navi2ch-bookmark-insert-subject i (car x))
      (setq i (1+ i)))))

(defun navi2ch-bookmark-delete ()
  "$B$=$N9T$r(B bookmark $B$+$i:o=|$9$k(B"
  (interactive)
  (save-excursion
    (beginning-of-line)
    (let ((item (navi2ch-bookmark-get-property (point))))
      (if item
          (let ((pair (assoc item navi2ch-bookmark-list))
                (buffer-read-only nil))
            (setq navi2ch-bookmark-list
                  (delq pair navi2ch-bookmark-list))
            (delete-region (point)
                           (save-excursion (forward-line) (point)))
            (navi2ch-bm-renumber))
        (message "Can't select this line!")))))

(defun navi2ch-bookmark-cut ()
  (interactive)
  (save-excursion
    (beginning-of-line)
    (let ((item (navi2ch-bookmark-get-property (point))))
      (if item
          (progn
            (push (assoc item navi2ch-bookmark-list)
                  navi2ch-bookmark-cut-stack)
            (navi2ch-bookmark-delete))
        (message "Can't select this line!")))))

(defun navi2ch-bookmark-yank ()
  (interactive)
  (let ((pair (pop navi2ch-bookmark-cut-stack)))
    (if pair
        (progn
          (if (eobp)
              (setcdr (last navi2ch-bookmark-list)
                      (list pair))
            (let ((list (member (assoc (navi2ch-bookmark-get-property (point))
                                       navi2ch-bookmark-list)
                                navi2ch-bookmark-list)))
              (setcdr list (copy-list list))
              (setcar list pair)))
          (let ((buffer-read-only nil))
            (navi2ch-bookmark-insert-subject 0 (car pair)))
          (navi2ch-bm-renumber))
      (message "stack is empty"))))

;; (defun navi2ch-bookmark-exit ()
;;   (interactive)
;;   (bury-buffer)
;;   (set-window-configuration navi2ch-bookmark-window-configuration))

(defun navi2ch-bookmark ()
  "bookmark $B$rI=<($9$k(B"
  (navi2ch-bookmark-mode)
  (navi2ch-bm-setup 'navi2ch-bookmark)
  (let ((buffer-read-only nil))
    (erase-buffer)
    (save-excursion
      (navi2ch-bookmark-insert-subjects))))

(defun navi2ch-bookmark-setup-menu ()
  (easy-menu-define navi2ch-bookmark-mode-menu
		    navi2ch-bookmark-mode-map
		    "Menu used in navi2ch-bookmark"
		    navi2ch-bookmark-mode-menu-spec)
  (easy-menu-add navi2ch-bookmark-mode-menu))

(defun navi2ch-bookmark-mode ()
  "\\{navi2ch-bookmark-mode-map}"
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'navi2ch-bookmark-mode)
  (setq mode-name "Navi2ch Bookmark")
  (setq buffer-read-only t)
  (use-local-map navi2ch-bookmark-mode-map)
  (navi2ch-bookmark-setup-menu)
  (run-hooks 'navi2ch-bookmark-mode-hook))

(defun navi2ch-bookmark-save-info ()
  (let ((list
         (mapcar
          (lambda (x)
            (let ((board (cdr (assq 'board (cdr x))))
                  (article (cdr (assq 'article (cdr x)))))
              (cons (car x)
                    (list (cons 'board
                                (list 
                                 (assq 'name board)
                                 (assq 'uri board)
                                 (assq 'id board)))
                          (cons 'article
                                (list
                                 (assq 'subject article)
                                 (assq 'artid article)))))))
          navi2ch-bookmark-list)))
    (navi2ch-save-info navi2ch-bookmark-file list)))

(defun navi2ch-bookmark-load-info ()
  (setq navi2ch-bookmark-list
        (navi2ch-load-info navi2ch-bookmark-file)))

(provide 'navi2ch-bookmark)
        
;;; navi2ch-bookmark.el ends here